@@ -211,7 +211,7 @@ int retry_writev(int fd, struct iovec *iov, int iovcnt) {
 		}
 
 		for (i = 0; i < iovcnt; i++) {
-			if (iov[i].iov_len > (unsigned) n) {
+			if ((int) iov[i].iov_len > n) {
 				iov[i].iov_base = (char *)iov[i].iov_base + n;
 				iov[i].iov_len -= n;
 				break;
@@ -226,3 +226,51 @@ int retry_writev(int fd, struct iovec *iov, int iovcnt) {
 		}
 	}
 }
+
+#ifndef HAVE_STRLCPY
+/* strlcpy -- copy string smartly.
+ *
+ * i believe/hope this is compatible with the BSD strlcpy(). 
+ */
+size_t saslauthd_strlcpy(char *dst, const char *src, size_t len)
+{
+	size_t n;
+
+	if (len <= 0) {
+		/* we can't do anything ! */
+		return strlen(src);
+	}
+
+	/* assert(len >= 1); */
+	for (n = 0; n < len-1; n++) {
+		if ((dst[n] = src[n]) == '\0') break;
+	}
+	if (n >= len-1) {
+		/* ran out of space */
+		dst[n] = '\0';
+		while(src[n]) n++;
+	}
+	return n;
+}
+#endif
+
+#ifndef HAVE_STRLCAT
+size_t saslauthd_strlcat(char *dst, const char *src, size_t len)
+{
+	size_t i, j, o;
+
+	o = strlen(dst);
+	if (len < o + 1)
+		return o + strlen(src);
+	len -= o + 1;
+	for (i = 0, j = o; i < len; i++, j++) {
+		if ((dst[j] = src[i]) == '\0') break;
+	}
+	dst[j] = '\0';
+	if (src[i] == '\0') {
+		return j;
+	} else {
+		return j + strlen(src + i);
+	}
+}
+#endif
